#!/usr/bin/env python

"""usage: asp_download [options]

    -d  use other config dir than ~/.autoSyncPodder
    -h  this help
    -i  interactive mode (you are asked before every
        download - use this after you added an new feed)
    -m  don't send mails - pipe all to stdout (default)
    -v  print the names of the downloaded files files

asp_sync is part of the autoSyncPodder, which allows you
to fully automatic download your podcasts and sync them with your
mp3 player. After the installation and configuration you just need
to plug your mp3 into your pc and the data is transfered without any
interaction.

The asp_download is in this setup responsible for downloading the
podcasts and putting them in the specified way into the sync
directory.
"""

#FIXME: Mount and Umount
#FIXME: sync

import sys
import os
import getopt
import feedparser
import ConfigParser
import subprocess
import urlparse
import string

import config_manager
import lock

class Downloader:
    """ we created a class so we can reuse the code in easily in an
        other programs
    """
    _configManager = None
    _noMails = None
    _interactive = None
    _verbose = None
    # feedID + command e.g. "lugradio", "none"
    _interactiveOverwrite = ["",""]
    
    def __init__(self, configManager, noMails, interactive, verbose):
        self._configManager = configManager
        self._noMails = noMails
        self._interactive = interactive
        self._verbose = verbose

    def _buildFeedsDict(self):
        """ builds a list out of the feeds directory """
        feedsList = []
        feedsDir = self._configManager.getFeedsDir()
        
        # run through the feeds dir, ignore everything thats not
        # an ini file, add the ini file as dict to the result
        for filename in os.listdir(feedsDir):
            feedID, ext = os.path.splitext(filename)
            if ext.lower() == ".ini":
                fullPath = os.path.join(feedsDir, filename)
                aConfig = ConfigParser.SafeConfigParser()
                aConfig.read(fullPath)
                aDict = dict(aConfig.items("General", True))
                aDict["id"] = feedID
                feedsList.append(aDict)
        return feedsList
    
    def _isNewEpisodeEntry(self, feedID, entryID, title):
        """ checks if an ID is already processed """
        fullPath = os.path.join(self._configManager.getStateDir(), feedID) + ".state"
        if not os.path.isfile(fullPath) or not entryID + "\n" in file(fullPath):
            if not self._interactive:
                return True

            #  obey the overwrite by the user
            if self._interactiveOverwrite[0] == feedID:
                if  self._interactiveOverwrite[1] == "all":
                    return True
                elif self._interactiveOverwrite[1] == "none":
                    self._addEpisodeEntry(feedID, entryID)
                    return False
            
            # we're in interactive mode so we ask the user
            answer = ""
            while (not answer in ("y", "n", "yes", "no", "all", "none")):
                sys.stdout.write("%s: download %s? (yes/no/all/none): " % (feedID, title))
                answer = raw_input()
            self._interactiveOverwrite = [feedID, answer]
                
            if answer in ("y", "yes"):
                return True
            
            # we never ask again
            self._addEpisodeEntry(feedID, entryID)
                
        return False

    def _addEpisodeEntry(self, feedID, entryID):
        """ add ID if not already added """
        fullPath = os.path.join(self._configManager.getStateDir(), feedID) + ".state"
        if not os.path.isfile(fullPath) or not entryID + "\n" in file(fullPath):
            file(fullPath,"a").write(entryID + "\n")
    
    
    def download(self):
        """ download all rss files from the feeds which were not
            already downloaded
        """
        #FIXME: after we know it works, put some error handling 
        #       into it
        
        # go through the podcasts ...
        for feedInfo in self._buildFeedsDict():
            # download the feeds
            feed = feedparser.parse(feedInfo["feed"]) 
            for entry in feed.entries:
                # currently we use the url of the file as entryID
                # later this should be configurable pre feed
                if not "enclosures" in entry or not self._isNewEpisodeEntry(feedInfo["id"], entry.enclosures[0].href, entry.title):
                    continue
                
                if self.downloadEnclosed(feedInfo, entry):
                    self._addEpisodeEntry(feedInfo["id"], entry.enclosures[0].href)
    
    def _formatFilename(self, filename, feedInfo, entry):
        """ format the Filename as wished by the user, per feed config """
        # prepare some stuff
        name, ext = os.path.splitext(filename)
        # replace chars (the first ones with the second ones)
        translation = string.maketrans(": /\*+|'" + '"', "__-----==")
        
        if feedInfo.get("replacefilenamewithtitle", "False").lower().strip() == "true":
            # just remove these chars while we perform the operation
            # on the episode title and we need to convert to Latin 15 and back
            name = entry.title.encode('iso8859_15').translate(translation, '?').decode('iso8859_15')
        
        if feedInfo.get("adddescriptiontoname", "False").lower().strip() == "true":
            name += "_" + entry.description.encode('iso8859_15').translate(translation, '?').decode('iso8859_15')
        
        if feedInfo.get("feednameasprefix", "False").lower().strip() == "true":
            name = feedInfo["id"] + "_" + name
        
        # return max 160 chars
        return name[:150] + ext[:10]
            
    def _call(self, args):
        """ this is a helper function which executes programs and
            throws a helpfull exception for the user if it didn't work
        """
        p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
        p.wait()
        if p.returncode < 0:
            raise IOError, "Child was terminated by signal %d - output: %s" % (0-p.returncode , p.stdout.read())
        elif p.returncode > 0:
            raise IOError, "Child returned %d - output: %s" % (p.returncode, p.stdout.read())

    def downloadEnclosed(self, feedInfo, entry):
        """ download the file to the working dir and move it than
            to the sync dir
        """
        s = os.path.basename(urlparse.urlsplit(entry.enclosures[0].href)[2])
        filename = self._formatFilename(s, feedInfo, entry)
        downloadedFilename = os.path.join(self._configManager.getDownloadDir(), filename)
        
        
        # download it, now!
        cmd = "/usr/bin/wget -T 30 -t 60 -c --progress=dot:mega -O".split()
        cmd.append(downloadedFilename)
        cmd.append(entry.enclosures[0].href)
        self._call(cmd)
        
        # check the download
        if "length" in entry.enclosures[0] and entry.enclosures[0].length.isdigit() and \
                os.stat(downloadedFilename).st_size < int(entry.enclosures[0].length):
            print "Error downloading %s (%s off %i bytes downloaded)" % (downloadedFilename,
                    entry.enclosures[0].length, os.stat(downloadedFilename).st_size)
            return False
            
        # now move it
        syncFilename = os.path.join(self._configManager.getSyncDir(), filename)
        os.rename(downloadedFilename, syncFilename)
        
        if self._verbose:
            print "%s downloaded." % filename.encode('iso8859_15')
        
        return True
        

# Print usage message and exit
def usage(*args):
    sys.stdout = sys.stderr
    print __doc__
    print 50*"-"
    for msg in args: print msg
    sys.exit(2)
    
# Main program: parse command line and start processing
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'd:himv')
    except getopt.error, msg:
        usage(msg)

    noMails = False
    configDir = None
    interactive = False
    verbose = False
    
    for o, a in opts:
        if o == '-h':
            print __doc__
            sys.exit()
        if o == '-d':
            configDir = a
        if o == '-i':
            interactive = True
        if o == '-m':
            noMails = True
        if o == '-v':
            verbose = True
    
    myConfigManager = config_manager.ConfigManager(configDir)

    # first lock other instances of me
    if not lock.test_and_lock(myConfigManager.getDownloadLockFile()):
        #FIXME: send mail, as this is an error!
        print "An other instance of asp_download is already running! Aborting"
        sys.exit(2)

    myDownloader = Downloader(myConfigManager,noMails, interactive, verbose)
    myDownloader.download()
    lock.unlock(myConfigManager.getDownloadLockFile())
    
if __name__ == '__main__':
    main()
