#!/usr/bin/env python

"""this class handles the config stuff and presents an easy interface for the other classes
"""

from ConfigParser import *
import os
import sys

defaultConfigDir = os.path.expanduser("~") + "/.autoSyncPodder/"
defaultConfigFile = "autoSyncPodder.ini"

class ConfigManager:
    _config = None
    _configFile = None
    _configDir = None
    def __init__(self, configDir = None):
        # this is needed as the caller can provide None as parameter
        if not configDir:
            configDir = defaultConfigDir
        
        configFile = os.path.join(configDir, "autoSyncPodder.ini")

        if not os.path.isfile(configFile):
            sys.stdout = sys.stderr
            print "Error: Cannot find config file %r" % configFile
            sys.exit(2)
 
        self._config = SafeConfigParser()
        self._config.read(configFile)
        self._configFile = configFile
        self._configDir = configDir

    def getConfigFile(self):
        return self._configFile
        
    def getConfigDir(self):
        return self._configDir
    
    def getFeedsDir(self):
        return os.path.join(self._configDir, "feeds")

    def getStateDir(self):
        return os.path.join(self._configDir, "state")
        
    # General
    
    # Download
    def getDownloadDir(self):
        return self._config.get("Download", "downloadDir")

    def getDownloadLockFile(self):
        return os.path.join(self._configDir, "asp_download.pid")
        
    # Sync    
    def getSyncDir(self):
        return self._config.get("Sync", "syncDir")
        
    def getPlayerDir(self):
        return self._config.get("Sync", "playerDir")

    def getSyncInfoFilename(self):
        return os.path.join(self.getPlayerDir(), ".autoSyncPodder")
    
    def getSyncInfoInProcessFilename(self):
        return os.path.join(self.getPlayerDir(), ".autoSyncPodder_inProcess")

    
    def getSyncLockFile(self):
        return os.path.join(self._configDir, "asp_sync.pid")

# Main program: parse command line and start processing
def main():
    myConfigManager = ConfigManager()
    print myConfigManager.getDownloadDir()
    print myConfigManager.getSyncDir()
    
if __name__ == '__main__':
    main()
